from faker_marketdata import MarketDataProvider

import unittest


class MarketDataTests(unittest.TestCase):

    def setUp(self):
        self.md = MarketDataProvider()

    def test_figi(self):
        figis = [self.md.figi() for _ in range(10000)]
        self.assertTrue(all(figi for figi in figis))

    def test_isin(self):
        isins = [self.md.isin() for _ in range(10000)]
        self.assertTrue(all(isin for isin in isins))

    def test_lei(self):
        leis = [self.md.lei() for _ in range(10000)]
        self.assertTrue(all(lei for lei in leis))

    def test_market_type(self):
        mt = [self.md.marketType() for _ in range(10000)]
        self.assertTrue(all(marketType for marketType in mt))

    def test_mic(self):
        mics = [self.md.mic() for _ in range(10000)]
        self.assertTrue(all(mic for mic in mics))

    def test_nsin(self):
        nsins = [self.md.nsin() for _ in range(10000)]
        self.assertTrue(all(nsin for nsin in nsins))

    def test_ric(self):
        rics = [self.md.ric() for _ in range(10000)]
        self.assertTrue(all(ric for ric in rics))

    def test_cusip (self):
        cusips = [self.md.cusip() for _ in range(10000)]
        self.assertTrue(all(cusip for cusip in cusips))

    def test_sedol(self):
        sedols = [self.md.sedol() for _ in range(10000)]
        self.assertTrue(all(sedol for sedol in sedols))

    def test_tickers(self):
        tickers = [self.md.ticker() for _ in range(10000)]
        self.assertTrue(all(ticker for ticker in tickers))

    """
    Tests integration with Faker
    """
    def test_faker_provider(self):
        from faker import Faker
        fake = Faker()
        fake.add_provider(MarketDataProvider())
        self.assertTrue(fake.sedol())

    def test_seed_fixes_random (self):
        from faker import Faker
        faker1 = Faker()
        faker2 = Faker()
        faker1.add_provider(MarketDataProvider(seed=23))
        faker2.add_provider(MarketDataProvider(seed=23))
        self.assertEqual(faker1.sedol(), faker2.sedol())


if __name__ == '__main__':
    unittest.main()



from faker.providers import BaseProvider
from faker import Faker
import random
import pycountry

_alphanumeric_alph = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
_alph = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
_cusip_alph = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ*@#'

class MarketDataProvider (BaseProvider):

    def __init__(self, seed=None):
        BaseProvider.__init__(self, random.Random(seed))
        self.rnd = random.Random(seed)

    # Validation extracted from https://github.com/arthurdejong/python-stdnum/blob/master/stdnum/isin.py
    def isin(self):
        """
        Generates a random, valid ISIN
        :return: a random ISIN with valid country code and validation digit
        """
        country_codes = map(lambda x: x.alpha_2, list(pycountry.countries))
        code = self.rnd.choice(list(country_codes)).upper()
        rnd_str = "".join(self.rnd.choices(_alphanumeric_alph[11:], k=9))
        number = code.upper() + rnd_str.zfill(9)
        number = ''.join(str(_alphanumeric_alph.index(n)) for n in number)
        number = ''.join( str((2, 1)[i % 2] * int(n)) for i, n in enumerate(reversed(number)))
        number = str((10 - sum(int(n) for n in number)) % 10)
        return "{}{}{}".format(code, rnd_str, number)

    def sedol(self):
        """
        Generates a random, valid SEDOL
        :return: SEDOL as a string
        """
        str_part = "".join(self.rnd.choices(_alphanumeric_alph[11:], k=6))
        weights = (1, 3, 1, 7, 3, 9)
        s = sum(w * _alphanumeric_alph.index(n) for w, n in zip(weights, str_part))
        number =  str((10 - s) % 10)
        return "{}{}".format(str_part, number)

    def mic(self):
        """
        Generates a random MIC
        :return: MIC as a string
        """
        return "".join(self.rnd.choices(_alphanumeric_alph[10:], k=4))

    def lei(self):
        lou = "".join(self.rnd.choices(_alphanumeric_alph, k=4))
        reserved = "00"
        org_id = "".join(self.rnd.choices(_alphanumeric_alph, k=12))
        partial_lei = "{}{}{}".format(lou, reserved, org_id)
        checksum = sum(list(map(lambda x: _alph.index(x) if x in _alph else 0, partial_lei))) % 98
        checksum = 98 - int(checksum)
        return "{}{}".format(partial_lei, checksum)

    def cusip(self):
        base_id = "".join(self.rnd.choices(_cusip_alph, k=8))
        check_char = ''.join(str((1, 2)[i % 2] * _cusip_alph.index(n)) for i, n in enumerate(base_id))
        check_char = str((10 - sum(int(n) for n in check_char)) % 10)
        return "{}{}".format(base_id, check_char)

    def ric(self):
        """
        Generates a random RIC string
        :return: a RIC with the format {three chars} {dot} {two chars}
        """
        ticker = "".join(self.rnd.choices(_alph, k=3))
        exchange = "".join(self.rnd.choices(_alph, k=2))
        return "{}.{}".format(ticker, exchange)

    def ticker (self):
        """
        Generates a random 4-char ticker
        :return: a random ticker ID
        """
        ticker = "".join(self.rnd.choices(_alph, k=4))
        return "{}".format(ticker)

    def nsin (self):
        code  = "".join(self.rnd.choices(_alphanumeric_alph, k=9))
        return "{}".format(code)

    def figi (self):
        head = "B" # """.join(self.rnd.choices(_alph, k=1))
        char2 = self.rnd.choices(_alph)[0]
        if head == "B":
            char2 = self.rnd.choice([c for c in _alph if c not in ['S','M']])
        elif head == "G":
            char2 = self.rnd.choice([c for c in _alph if c not in ['G','B', 'H']])
        elif head == "K":
            char2 = self.rnd.choice([c for c in _alph if c != 'Y'])
        elif head == "V":
            char2 = self.rnd.choice([c for c in _alph if c != 'G'])
        char3 = "G"
        tail = "".join(self.rnd.choices(_alphanumeric_alph, k=7))
        id = "{}{}{}{}".format(head, char2, char3, tail)
        number = ''.join(str(_alphanumeric_alph.index(n) * (1, 2)[i % 2]) for i, n in enumerate(id[:11]))
        number = str((10 - sum(int(n) for n in number)) % 10)
        return "{}{}".format(id, number)

    def marketType (self):
        return self.rnd.choice(["bond", "commodity", "derivatives", "forex", "otc", "pe", "spot", "stock"])



from setuptools import setup, find_packages

setup(
    name="faker_marketdata",
    version="0.1",
    author="Ruben Afonso",
    author_email="rbfrancos@gmail.com",
    description="Sample market data for Faker",
    url="https://github.com/rubenafo/faker_marketdata",
    keywords = ["faker", "testdata", "financedata", "python", "marketdata", "isin", "sedol", "cusip",
                "ticker", "figi"],
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable"
    ],
    python_requires='>=3.6',
)